"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useNearbyErrorCallback = exports.useNearbySearch = exports.useNearbySubscription = exports.useNearbyPublication = void 0;
const react_1 = require("react");
const index_1 = require("index");
/**
 * Publish a simple message and return the current status of the nearby API.
 *
 * Also calls `checkBluetoothAvailability()` and `checkBluetoothPermission()`.
 * @param config The Nearby API configuration object to use.
 * @param message The message to publish
 * @returns The current status of the Nearby API
 */
function useNearbyPublication(config, message) {
    const [nearbyStatus, setNearbyStatus] = react_1.useState('connecting');
    react_1.useEffect(() => {
        const start = async () => {
            try {
                const available = await index_1.checkBluetoothAvailability();
                if (!available) {
                    setNearbyStatus('unavailable');
                    return;
                }
                const permission = await index_1.checkBluetoothPermission();
                if (!permission) {
                    setNearbyStatus('denied');
                    return;
                }
                await index_1.connect(config);
                await index_1.publish(message);
                setNearbyStatus('published');
            }
            catch (e) {
                setNearbyStatus('error');
            }
        };
        start();
        const removeListener = index_1.addOnErrorListener((kind, message) => {
            console.log(`[NEARBY] Error: ${kind}: ${message}`);
            setNearbyStatus('error');
        });
        return () => {
            removeListener();
            index_1.unpublish();
            index_1.disconnect();
            setNearbyStatus('disconnected');
        };
    }, [setNearbyStatus, config, message, setNearbyStatus]);
    return nearbyStatus;
}
exports.useNearbyPublication = useNearbyPublication;
function reducer(messages, payload) {
    const removeIndices = payload.removeMessages?.map((removeMessage) => messages.findIndex((message) => removeMessage === message));
    removeIndices?.forEach((i) => {
        if (i > -1)
            messages.splice(i, 1);
    });
    payload.addMessages?.forEach((u) => {
        if (messages.indexOf(u) === -1)
            messages.push(u);
    });
    return [...messages];
}
/**
 * Subscribe to nearby messages and return an instance of the `SubscriptionState` object.
 *
 * Also calls `checkBluetoothAvailability()` and `checkBluetoothPermission()`.
 * @param config The Nearby API configuration object to use.
 * @returns A state of all nearby messages
 */
function useNearbySubscription(config) {
    const [nearbyMessages, dispatch] = react_1.useReducer(reducer, []);
    const [nearbyStatus, setNearbyStatus] = react_1.useState('connecting');
    const messageFound = react_1.useCallback((message) => {
        dispatch({ addMessages: [message] });
    }, [dispatch]);
    const messageLost = react_1.useCallback((message) => {
        dispatch({ removeMessages: [message] });
    }, [dispatch]);
    react_1.useEffect(() => {
        const start = async () => {
            try {
                const available = await index_1.checkBluetoothAvailability();
                if (!available) {
                    setNearbyStatus('unavailable');
                    return;
                }
                const permission = await index_1.checkBluetoothPermission();
                if (!permission) {
                    setNearbyStatus('denied');
                    return;
                }
                await index_1.connect(config);
                await index_1.subscribe(messageFound, messageLost);
                setNearbyStatus('subscribed');
            }
            catch (e) {
                setNearbyStatus('error');
            }
        };
        start();
        const removeListener = index_1.addOnErrorListener((kind, message) => {
            console.log(`[NEARBY] Error: ${kind}: ${message}`);
            setNearbyStatus('error');
        });
        return () => {
            removeListener();
            index_1.unpublish();
            index_1.disconnect();
            setNearbyStatus('disconnected');
        };
    }, [config, messageFound, messageLost, setNearbyStatus]);
    return { nearbyMessages, nearbyStatus };
}
exports.useNearbySubscription = useNearbySubscription;
/**
 * Search for a specific message using the nearby messages API. Returns an instance of the `SearchState` interface.
 *
 * Also calls `checkBluetoothAvailability()` and `checkBluetoothPermission()`.
 * @param config The Nearby API configuration object to use.
 * @param searchFor The string to perform the nearby search for
 * @returns A state whether the message has been found or not.
 */
function useNearbySearch(config, searchFor) {
    const [isNearby, setIsNearby] = react_1.useState(false);
    const [nearbyStatus, setNearbyStatus] = react_1.useState('connecting');
    const messageFound = react_1.useCallback((message) => {
        if (message === searchFor)
            setIsNearby(true);
    }, [searchFor, setIsNearby]);
    const messageLost = react_1.useCallback((message) => {
        if (message === searchFor)
            setIsNearby(false);
    }, [searchFor, setIsNearby]);
    react_1.useEffect(() => {
        const start = async () => {
            try {
                const available = await index_1.checkBluetoothAvailability();
                if (!available) {
                    setNearbyStatus('unavailable');
                    return;
                }
                const permission = await index_1.checkBluetoothPermission();
                if (!permission) {
                    setNearbyStatus('denied');
                    return;
                }
                await index_1.connect(config);
                await index_1.subscribe(messageFound, messageLost);
                setNearbyStatus('subscribed');
            }
            catch (e) {
                setNearbyStatus('error');
            }
        };
        start();
        const removeListener = index_1.addOnErrorListener((kind, message) => {
            console.log(`[NEARBY] Error: ${kind}: ${message}`);
            setNearbyStatus('error');
        });
        return () => {
            removeListener();
            index_1.unsubscribe();
            index_1.disconnect();
            setNearbyStatus('disconnected');
        };
    }, [config, messageFound, messageLost, setNearbyStatus]);
    return { isNearby, nearbyStatus };
}
exports.useNearbySearch = useNearbySearch;
/**
 * Add an error listener which automatically disposes when the component unmounts.
 * @param callback The function to call when an error occurs.
 */
function useNearbyErrorCallback(callback) {
    react_1.useEffect(() => index_1.addOnErrorListener(callback), [callback]);
}
exports.useNearbyErrorCallback = useNearbyErrorCallback;
